;Routines for SysHacker

__vputs:					;These work from any ROM page
	call $28ee
	.dw _vputs
	.db $0d

__vputmap:
	call $28ee
	.dw _vputmap
	.db $0d

__cphlde:
	call $28ee
	.dw _cphlde
	.db $0d

Convert_Char:				;Convert char code in a to number in a (call)
	cp L0
	jr c,Convert_Char_Error
	cp L9+1
	jr c,Convert_Char_Num
	cp LcapA
	jr c,Convert_Char_Error
	cp LcapF+1
	jr c,Convert_Char_Letter
	cp La
	jr c,Convert_Char_Error
	cp Lf+1
	jr nc,Convert_Char_Error
Convert_Char_Small_Letter:
	sub La-$0a
	ret
Convert_Char_Letter:
	sub LcapA-$0a
	ret
Convert_Char_Num:
	sub L0
	ret
Convert_Char_Error:
	pop hl
	jp Value_Error

Call_HL:					;Call address in hl, then return to event handler (jp)
	ld (Patch1),hl
Patch1	equ	$+1
	call $0000
	jp Event_Handler

Clear_Wind:					;Clear area of screen where data/prompts are displayed (call)
	ld hl,$fc00+(7*16)
	ld de,$fc00+(7*16)+1
	ld (hl),0
	ld bc,50*16-1
	ldir
	ret

Pause:						;Wait for a key to be pressed (call)
	call _getky
	or a
	jr z,Pause
	ret

Get_Addr_From_Table:		;Find entry a in table at (hl), return address in hl (call)
	add a,a
	ld c,a
	ld b,0
	add hl,bc
	jp _ldhlind

Copy_To_Buffer:				;Copy data starting at (Top_Addr) to (Data_Buffer)
							;with ROM page in ROM_Page and RAM page in RAM_Page loaded (call)
	ld a,(ROM_Page)			;What ROM page are we supposed to be on?
	out (5),a				;Load it into the proper area
	ld a,(RAM_Page)			;Get the RAM page we're supposed to be on
	set 6,a					;and load that too
	out (6),a
	ld hl,(Top_Addr)		;Now see where we should be getting the data from
	ld de,Data_Buffer		;and copy it to the buffer
	ld bc,Buffer_Len
	ldir
	dec hl
	ld (Bot_Addr),hl
;	jp Reset_Pages		;Restore ROM page and RAM page

Reset_Pages:							;Load correct RAM/ROM pages (call)
	ld a,$0d
	out (5),a
	ld a,$41
	out (6),a
	ret

Draw_Bit_newline:			;call Draw_Bit, call _vnewline (call)
	call Draw_Bit
	jr _vnewline

_vputs_newline:				;call _vputs, call _vnewline (call)
	call _vputs

_vnewline:					;Moves _penXXX cursor to start of next line (call)
	push hl
	ld hl,_penRow
	ld a,(hl)
	add a,Line_Spacing
	ld (hl),a
	dec hl
Patch2	equ	$+1
	ld (hl),0
	pop hl
	ret

Disp_HL_Hex:				;Draw hl as a hex number in small font (call)
	ld a,h
	call Draw_Hex_Byte
	ld a,l
Draw_Hex_Byte:				;Draw a as 2 hex digits in small font (call)
	push af
	and $f0					;Get upper nibble
	call _shracc            ;move into lower position
	call Draw_Hex_Digit     ;and display
	pop af                  ;Get lower nibble
	and $0f                 ;then display
Draw_Hex_Digit:	            ;Draw a as hex digit
	cp 10                   ;Is above 9?
	jr nc,Draw_Hex_Digit_L  ;If so, draw A-F, doesn't need to be converted to ASCII
Draw_Hex_Digit_N:
	add a,L0                ;Otherwise is number, add to ASCII code for 0 to get ASCII code for number
Patch10 equ $+1
	jp _vputmap             ;Now draw
;	push bc
;	pop hl
;	jp (hl)
Draw_Hex_Digit_L:
	add a,LcapA-$0a
Patch9 equ $+1
	jp _vputmap
;	push bc
;	pop hl
;	jp (hl)

Draw_Hex_Byte_2:				;Draw a as 2 hex digits in large font (call)
	push hl
	ld hl,_putc
	ld (Patch10),hl
	ld (Patch9),hl
	call Draw_Hex_Byte
	ld hl,_vputmap
	ld (Patch10),hl
	ld (Patch9),hl
	pop hl
	ret

Draw_Bin_Byte:				;Draw a as 8-bit binary num in small font (call)
	bit 7,a
	call Draw_Bit
	bit 6,a
	call Draw_Bit
	bit 5,a
	call Draw_Bit
	bit 4,a
	call Draw_Bit
	bit 3,a
	call Draw_Bit
	bit 2,a
	call Draw_Bit
	bit 1,a
	call Draw_Bit
	bit 0,a
Draw_Bit:
	push af
	jr z,Draw_Bin_0
Draw_Bin_1:
	ld a,L1
	jr Draw_Bin
Draw_Bin_0:
	ld a,L0
Draw_Bin:
	call _vputmap
	pop af
	ret

Refresh_Page_Disp:			;Draw currently selected RAM/ROM pages at top of screen (call)
	set textInverse,(iy+textflags)
	ld hl,RAM_Pos
	ld (_penCol),hl
	ld hl,RAM_Str
	call _vputs
	ld a,(RAM_Page)
	call Draw_Hex_Digit
	ld hl,ROM_Pos
	ld (_penCol),hl
	ld hl,ROM_Str
	call _vputs
	ld a,(ROM_Page)
	call Draw_Hex_Digit
	res textInverse,(iy+textflags)
	ret

Go_Back_To_Prev_State:					;Go back to the previous state (jp)
	ld hl,(Ret_Addr)
	jp (hl)

Get_Hex_Num:				;Get hex number from user with a digits, b=1 if +, - allowed, prompt at (hl), store in hl, b, a=0 if abs, a=1 if +, a=2 if -
	push bc
	ld (Patch7),a
	push hl
	ld hl,edit_buffer		;clear edit buffer
	ld de,edit_buffer+1
	ld bc,5
	ld (hl),$30
	ldir
	call Draw_Window		;Draw window
	ld hl,Input_Pos			;set cursor position
	ld (_penCol),hl
	pop hl
	call _vputs				;and display prompt
	pop bc
	djnz Get_Hex_Num_Skip
	call _getkey
	cp kAdd
	jr z,Get_Hex_Num_Plus
	cp kSub
	jr z,Get_Hex_Num_Minus
	ld b,a
	xor a
	ld (Patch6),a
	ld a,b
	call Get_Hex_Num_Init
	jr Test_Keys
Get_Hex_Num_Plus:
	ld a,1
	ld (Patch6),a
	ld a,LplusSign
	call _vputmap
	jr Get_Hex_Num_Skip
Get_Hex_Num_Minus:
	ld a,2
	ld (Patch6),a
	ld a,Ldash
	call _vputmap
Get_Hex_Num_Skip:
	call Get_Hex_Num_Init
Get_Hex_Num_Loop:
	push hl
	call _getkey
	pop hl
Test_Keys:
	cp kExit
	jr z,Get_Hex_Num_Exit
	cp kDel
	jr nz,Test_More_Keys
Get_Hex_Num_Delete:
	ld de,edit_buffer
	call _cphlde
	jr z,Get_Hex_Num_Loop
	dec hl
	push hl
	ld hl,_penCol
	ld a,(hl)
	sub 4
	ld (hl),a
	call _ldhlind
	push hl
	ld hl,Del_Str
	call _vputs
	pop hl
	ld (_penCol),hl
	pop hl
	jr Get_Hex_Num_Loop
Get_Hex_Num_Exit:
	pop hl
	ld (iy+shiftflags),0
	jp Go_Back_To_Prev_State
Test_More_Keys:
	cp k0
	jr c,Test_More_Keys_2
	cp k9+1
	jr nc,Test_More_Keys_2
	sub k0-L0
	push af
	push hl
	call _vputmap
	pop hl
	pop af
	jr Insert_Char
Test_More_Keys_2:
	cp kCapA
	jr c,Test_More_Keys_3
	cp kCapF+1
	jr nc,Test_More_Keys_3
	sub kCapA-LcapA
	push af
	push hl
	call _vputmap
	pop hl
	pop af
	jr Insert_Char
Test_More_Keys_3:
	cp ka
	jr c,Get_Hex_Num_Loop
	cp kf+1
	jr nc,Get_Hex_Num_Loop
	sub ka-La
	push af
	push hl
	call _vputmap
	pop hl
	pop af
Insert_Char:
	ld (hl),a
	inc hl
	push hl
	or a
	sbc hl,bc
	pop hl
	jr z,End_Get_Hex
	jr Get_Hex_Num_Loop
End_Get_Hex:
	ld a,(edit_buffer)
	call Convert_Char
	call _shlacc
	push af
	ld a,(edit_buffer+1)
	call Convert_Char
	ld b,a
	pop af
	or b
	ld h,a
	ld a,(edit_buffer+2)
	call Convert_Char
	call _shlacc
	push af
	ld a,(edit_buffer+3)
	call Convert_Char
	ld b,a
	pop af
	or b
	ld l,a
	ld a,(edit_buffer+4)
	call Convert_Char
	call _shlacc
	push af
	ld a,(edit_buffer+5)
	call Convert_Char
	ld b,a
	pop af
	or b
	ld b,a
Patch6	equ	$+1
	ld a,0
	ld (iy+shiftflags),0
	ret
Get_Hex_Num_Init:
	ld de,edit_buffer
	push de
	pop hl
Patch7	equ	$+1
	ld c,0
	ld b,0
	add hl,bc
	push hl
	pop bc
	ex de,hl
	ret

Draw_Window:							;Draw fixed-size window on screen (call)
	ld hl,Win_Start_Addr
	ld b,Win_Width
Draw_Window_Top:
	ld (hl),$ff
	inc hl
	djnz Draw_Window_Top
	ld b,Win_Height
Draw_Window_Body:
	ld de,16-Win_Width
	add hl,de
	ld (hl),$80
	inc hl
	push bc
	ld b,Win_Width-2
Draw_Window_Body_Loop:
	ld (hl),0
	inc hl
	djnz Draw_Window_Body_Loop
	pop bc
	ld (hl),$01
	inc hl
	djnz Draw_Window_Body
	ld de,16-Win_Width
	add hl,de
	ld b,Win_Width
Draw_Window_Bot:
	ld (hl),$ff
	inc hl
	djnz Draw_Window_Bot
	ret

;Very heavily based on one of Jonah Cohen's search routines
Search:									;Search 1 RAM/ROM page, start addr in hl, length to search in bc
	dec hl
Find_First_Byte:
	ld de,Str_Buf						;Get first byte to search for
	ld a,(de)
	inc hl
	cpir								;Go until counter runs out or found
Found_First_Byte:
	dec hl
	cp (hl)								;If it matches, test next bytes
	jr nz,Not_Found						;If it doesn't, the counter has timed out-byte not found!
	push de
	push hl
	ld de,Str_Buf						;If found string in search string buffer, skip
	call __cphlde
	pop hl
	pop de
	jr z,Find_First_Byte
	ld a,(size)							;Init loop counter
	dec a
	jr z,Found							;If the data was only 1 byte long, then we've found it!
	push bc								;(save progress indic)
	ld b,a
	ld de,Str_Buf+1						;and point to start of untested data
	inc hl
Look_At_Next:
	ld a,(de)							;Look at next byte
	cp (hl)								;Is it equal in search string and mem?
	jr z,Look_At_Next_Continue			;If not, search for next occurrence of first byte
	pop bc								;(restore progress counter first)
	jr Find_First_Byte
Look_At_Next_Continue:
	inc de
	inc hl
	djnz Look_At_Next					;Go until all data in buffer has been tested
	ld a,(size)							;If it's all been tested, then it matches!
	ld e,a								;Subtract length of data from pointer to get real start of found string
	ld d,0
	or a
	sbc hl,de
	pop bc								;Remove saved progress counter from stack to prevent crash
Found:
	or a
	ret

Not_Found:
	scf
	ret

size		equ	$c9fc
page		equ	$c9fd

Calc_Search_Length:						;Return length of search in bc, input search start addr in hl, search end in de
	push hl
	ex de,hl
	sbc hl,de
	push hl
	pop bc
	pop hl
	ret

;/////////////////////////////////////////////MENUS/////////////////////////////////////////////////

Do_Menu:					;Draw menu on screen with text at (hl), address table at (de), number of entries in b (call)
	ld (Patch4),de			;Save address table pointer
	ld a,(_penCol)
	ld (Patch2),a
	ld a,b
	inc a
	ld (Patch3),a			;When key is pressed, make sure that it's in range
	push de
Do_Menu_Draw_Loop:
	push bc
	call _vputs				;Draw each line
	pop bc
	call _vnewline			;then go to next line
	djnz Do_Menu_Draw_Loop	;do this for all the entries
	pop de
	ld a,stMenu				;then show that it's in a menu
	ld (Cur_State),a
	ret

Menu_Key_Handler:
	ld a,(Key_Val)			;Look at keypress
	cp K_1					;Get the actual number that was pressed
	jr nz,Menu_0
	xor a
	jr End_Menu_Key
Menu_0:
	cp K_2
	jr nz,Menu_1
	ld a,1
	jr End_Menu_Key
Menu_1:
	cp K_3
	jr nz,Menu_2
	ld a,2
	jr End_Menu_Key
Menu_2:
	cp K_4
	jr nz,Menu_3
	ld a,3
	jr End_Menu_Key
Menu_3:
	cp K_5
	jr nz,Menu_4
	ld a,4
	jr End_Menu_Key
Menu_4:
	cp K_6
	jr nz,Menu_5
	ld a,5
	jr End_Menu_Key
Menu_5:
	cp K_7
	jr nz,Menu_6
	ld a,6
	jr End_Menu_Key
Menu_6:
	cp K_8
	jr nz,Menu_7
	ld a,7
	jr End_Menu_Key
Menu_7:
	cp K_9
	ret nz
	ld a,8
End_Menu_Key:
Patch3	equ	$+1
	cp 0
	ret nc					;Don't allow a keypress higher than the number of entries
Patch4	equ	$+1
	ld hl,$0000				;Add that to the start of the address table
	call Get_Addr_From_Table	;get the proper address
	jp (hl)					;and go to it

.end
